#!/usr/bin/env python3
from typing import Optional

from dm_control import suite
from envs.tasks import ball_in_cup, cheetah, fish, hopper, pendulum, reacher, walker

suite.ALL_TASKS = suite.ALL_TASKS + suite._get_tasks("custom")
suite.TASKS_BY_DOMAIN = suite._get_tasks_by_domain(suite.ALL_TASKS)

from torchrl.envs import (
    DMControlEnv,
    DMControlWrapper,
    GymEnv,
    StepCounter,
    TransformedEnv,
)
from torchrl.envs.transforms import (
    CatFrames,
    CatTensors,
    Compose,
    DoubleToFloat,
    RenameTransform,
    Resize,
    RewardSum,
    ToTensorImage,
    TransformedEnv,
)
from torchrl.record import VideoRecorder
from torchrl.record.loggers.csv import CSVLogger


def make_env(
    env_name: str,
    task_name: Optional[str] = None,
    from_pixels: bool = True,
    frame_skip: int = 2,
    pixels_only: bool = False,
    record_video: bool = False,
    device: str = "cpu",
):
    if env_name == "cup":
        env_name = "ball_in_cup"
    try:
        env = DMControlEnv(
            env_name=env_name,
            task_name=task_name,
            from_pixels=from_pixels or record_video,
            frame_skip=frame_skip,
            pixels_only=pixels_only,
            device=device,
        )
    except RuntimeError:
        env = suite.load(env_name, task_name)
        env = DMControlWrapper(
            env,
            from_pixels=from_pixels or record_video,
            frame_skip=frame_skip,
            pixels_only=pixels_only,
            device=device,
        )
    if not pixels_only:
        # Put "position"/"velocity"/"orientation" into "observation"
        obs_keys = [key for key in env.observation_spec.keys()]
        if from_pixels or record_video:
            obs_keys.remove("pixels")
        # print(f"Putting obs_keys: {obs_keys} into 'state'")
        env = TransformedEnv(env, CatTensors(in_keys=obs_keys, out_key="observation"))

        # env = TransformedEnv(
        #     env,
        #     CatTensors(in_keys=obs_keys, out_key=("observation", "state")),
        # )
    return env
